// C Header File
// Created 27/09/02; 18:20:13

//--------------------------------------------------------------------------
// <install_sub.h>
// Sub-routines used by install.c
//--------------------------------------------------------------------------

#ifndef __INSTALL_SUB_H__
#define __INSTALL_SUB_H__


//==========================================================================
// Scan the linked list of the ACBs, and free all the fake-ACBs and there
// corresponding object frame.
//==========================================================================
void FreeCreatedACB_and_Frames (void) {
	AppID CurrentACB = OO_firstACB;
	
	while (CurrentACB) {
		ACB *ACB_Ptr = HeapDeref (CurrentACB);
		if (ACB_Ptr->publicstorage == ACB_PRGM_MAGIC 
		 || ACB_Ptr->publicstorage == ACB_REFRESH_DT_MAGIC) {     // it is a fake ACB
			RemoveACBAndOFrame (ACB_Ptr);
			CurrentACB = ACB_Ptr->next;
			HeapFree (ACB_Ptr->myID);
		}     // if (ACB_Ptr-> ...
		else
			CurrentACB = ACB_Ptr->next;
	}     // while (CurrentACB)
} // FreeCreatedACB()


//==========================================================================
// Returns a pointer to the first folder of the Desktop Folders Structure
// ('All' is NOT considered as a folder)
// Returns NULL if not found.
//==========================================================================
static inline void *GetDT_FoldersPtr (void) {
	void *FoldersPtr = DT_FOLDERS_MIN_AD;
	AppID FirstHandle;
	BOOL found = FALSE;
	DT_FolderStruct DT_FolderSt;   // just for sizeof()
	const char *AllCatName = XR_stringPtr (XR_DEF_CATG_1_NAME); // will work with any language

	while (FoldersPtr < DT_FOLDERS_MAX_AD) {
		
		// first we search for the folder 'All'
		if (strcmp (AllCatName, FoldersPtr)) {   // not 'All'
			FoldersPtr += 2;     // must be at an even address
			continue;
		}
		FoldersPtr += sizeof (DT_FolderSt.folder_name);		
		if (*((short*)FoldersPtr) != 0) {     // check 2 '\0'
			FoldersPtr -= sizeof (DT_FolderSt.folder_name - 2); // 2 bytes after AllCatName
			continue;
		}
		FoldersPtr += 2;

		// Now FoldersPtr points theorically to an handle. Really ?
		if (*(unsigned short*)FoldersPtr > MAX_HANDLES_NUM)
			continue;
		FirstHandle = *(AppID*)HeapDeref (*(HANDLE*)FoldersPtr);		
		// *(HANDLE*)FoldersPtr is the handle of the list of the ACB handles
		// of the flash apps in the 'All' folder. Is the first handle really
		// an handle ?
		if (FirstHandle > MAX_HANDLES_NUM)
			continue;
		// Is it really an handle of an ACB ? We check if the myID member is 
		// correct.
		if ( ((ACB*)HeapDeref(FirstHandle))->myID == FirstHandle) {
			// Ok, we have found the folders !
			found = TRUE;
			break;
		}

	} // while

	if (found == FALSE)
		return NULL;
	// FoldersPtr points to the handle of the list of 'All'
	// We add 2 to make it point to the name of the next folder.
	return (FoldersPtr + 2);
	
}     // GetDT_FoldersPtr()


//==========================================================================
// Returns TRUE if the Text Viewer is found.
// The Text Viewer is an ASM or TI-BASIC program of name TXT_VIEWER_NAME,
// getting the name of the text file to open between brackets after its
// name, in the form of a string.
//==========================================================================
static inline BOOL FindTxtViewer (void) {
	if (!FindFile (TXT_VIEWER_NAME, NULL, FALSE, FALSE, NULL, NULL))
		return FALSE;
	return TRUE;
}     // FindTextViewer()


//==========================================================================
// Returns the size in bytes of a list of Apps in a Desktop folder.
// AppsListPtr must point to the beginning of the list.
// The null word at the end of the list is taken into account.
//==========================================================================
static inline unsigned short GetAppsListSize (void *AppsListPtr) {
	unsigned short size = sizeof(AppID);
	while (*((AppID*)AppsListPtr)++)
		size += sizeof(AppID);
	return (size);
}     // GetAppsListSize()


//==========================================================================
// Returns the number of Apps of a list of Apps in a Desktop Folder.
// AppsListPtr must point to the beginning of the list.
//==========================================================================
static inline unsigned short GetAppsNumberInFolder (void *AppsListPtr) {
	unsigned short number = 0;
	while (*((AppID*)AppsListPtr)++)
		number++;
	return (number);
}     // GetAppsNumberInFolder()


#endif     // #ifndef __INSTALL_SUB_H__

